/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: midiMultifile.c,v 1.2 2005/08/08 09:56:26 koehler Exp $
 * $Log: midiMultifile.c,v $
 * Revision 1.2  2005/08/08 09:56:26  koehler
 * Removed GPL-blurb (will be added by Makefile.export)
 *
 * Revision 1.1  2005/01/19 14:17:42  jaffe
 * new heidelberg additions
 *
 * Created:     Tue Aug 19 14:30:17 2003 by Koehler@sun47
 * Last change: Mon Aug  8 11:55:23 2005
 *
 * This file is Hellware!
 * It may send you and your data straight to hell without further notice.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include "midiMultifile.h"

char** strSplit (char *string, int *nString) {
   char *blank = " ";
   char *oldString;
   int   nS, iS ;
   char *tok;
   char *stringCopy;
   char **output;

   *nString = 0;
   stringCopy = strdup(string);
   oldString = stringCopy;
   tok = strtok(oldString, blank);
   if (tok == NULL) return NULL;
   oldString = NULL;
   for (nS = 0; tok != NULL; nS++) {
      tok = strtok(oldString, blank);
   }
   *nString = nS;
   output = (char **)malloc(nS * sizeof(char *));
   strcpy (stringCopy, string);
   oldString = stringCopy;
   blank = " ";
   for (iS=0; iS<nS; iS++) {
      tok = strtok(oldString, blank);
      oldString = NULL;
      output[iS]= strdup(tok);
   }
   free(oldString);
   return output;
} /* end of strSplit */


void killFileList(char **fileList, int nFiles) {
   int iFile;
   for (iFile=0; iFile<nFiles;iFile++) {free(fileList[iFile]);}
   free(fileList);
   return;
} /* end of killFileList */


/*************************************************************************/
/** code to read multi-file datasets transparently			**/
/*************************************************************************/

int multifile_close(struct multifile *MFile, int *ierr)
{
  if( MFile->open_no >= 0 && MFile->open_file != NULL)
	fits_close_file( MFile->open_file, ierr);
  free(MFile->nframes);
  killFileList( MFile->filelist,MFile->nFiles);
  free(MFile);
  return *ierr;
}

struct multifile *multifile_open(char *files, int *ierr)
{
  struct multifile *MFile;
  int i;

  if ((MFile= malloc(sizeof(struct multifile))) == NULL)
  {	*ierr = MEMORY_ALLOCATION;  return NULL; }

  MFile->filelist = strSplit(files,&(MFile->nFiles));
  MFile->nframes  = NULL;
  MFile->totframes= 0;
  MFile->open_file= NULL;
  MFile->open_no  = -1;

  if ((MFile->nframes= malloc(sizeof(long) * MFile->nFiles)) == NULL)
  {	multifile_close( MFile, ierr);
	*ierr = MEMORY_ALLOCATION;
	return NULL;
  }
  for (i=0;  i<MFile->nFiles;  ++i)
  {
    MFile->open_no = i;
    fits_open_file( &(MFile->open_file), MFile->filelist[i], READONLY, ierr);
    MFile->idata = open_imaging_data_table( MFile->open_file, ierr);
    fits_get_num_rows( MFile->open_file, &(MFile->nframes[i]), ierr);
    if (*ierr)
    {	multifile_close( MFile, ierr);	return NULL;  }

    MFile->totframes += MFile->nframes[i];
    printf("%s has %ld rows\n",MFile->filelist[i],MFile->nframes[i]);
    kill_imaging_data_table_def( MFile->idata,1);  MFile->idata= NULL;
    fits_close_file( MFile->open_file, ierr);
  }
  MFile->open_no = -1;
  return MFile;
}

int multifile_rd_img_data_tbl(struct multifile *MFile,
	imaging_data_row *data, long wantRow, int *ierr)
{
  int iFile= 0, i;
  long iRow= wantRow;

  /** figure out in which file the row wanted is **/
  /** remeber that rows start at 1, so file i contains rows 1...nframes[i] **/
  for( iFile= 0;  iFile < MFile->nFiles;  ++iFile)
  {
    if( iRow <= MFile->nframes[iFile])  break;
    iRow -= MFile->nframes[iFile];
  }
  /** do we have the file? **/
  if( iFile >= MFile->nFiles)
  {  *ierr = END_OF_FILE;  return *ierr; }

  /** is the file already open? **/
  if( iFile != MFile->open_no)
  {
    if( MFile->open_no >= 0)
    { if( MFile->idata)
      { kill_imaging_data_table_def( MFile->idata,1);  MFile->idata= NULL; }
      fits_close_file( MFile->open_file, ierr);
    }
    fprintf(stderr,"File %d = %s\n",iFile,MFile->filelist[iFile]);
    MFile->open_no= iFile;
    fits_open_file( &(MFile->open_file), MFile->filelist[iFile], READONLY, ierr);
    MFile->inDetectorDef= read_imaging_detector_table( MFile->open_file, ierr);
    MFile->idata = open_imaging_data_table( MFile->open_file, ierr);
    /* suppress autoscaling to avoid overflow */
    for( i=0;  i < MFile->idata->nregion; i++)
	fits_set_tscale( MFile->open_file, 1+MFile->idata->data_col[i], 1.0, 0.0,ierr);
  }
#if DEBUG > 1
  printf("want row %ld => file %d, row %ld\n",wantRow,iFile,iRow);
#endif
  return read_imaging_data_table( MFile->open_file, MFile->idata, data, iRow, ierr);
}
